#!/usr/bin/env python3
# coding: utf-8
# -*- coding: utf-8 -*-
# A bridge to set rudder node into netbox

import requests, os, random, sys, time, json, logging, argparse, re, shutil

from logging.handlers import RotatingFileHandler
from pprint import pprint
from utils.config import get_parameter, init_log
from utils.netbox import *

config_file         = "config.txt"
secrets_filepath    = get_parameter("secrets_filepath", config_file)
rudder_url          = get_parameter("rudder_url", config_file)
netbox_url          = get_parameter("netbox_url", config_file)
log_filepath        = get_parameter("log_filepath", config_file)
log                 = init_log(log_filepath)


def get_nodes(config):
    rudder_token        = get_parameter("rudder_token", secrets_filepath)
    nodes = requests.get(rudder_url+'/rudder/api/latest/nodes', headers={'X-API-Token': rudder_token})
    for node in nodes.json()['data']['nodes']:
        if check_if_node_exist(node['hostname'], config) == 0:
            add_netbox_server(node['hostname'], 1, 1, node['ram'], 10, node['machine']['type'], config)
        else:
            full_nodes_info = requests.get(rudder_url+'/rudder/api/latest/nodes/'+node['id']+'?include=full', headers={'X-API-Token': rudder_token})
            for full_node_info in full_nodes_info.json()['data']['nodes']:
                if node['machine']['type'] == "Virtual":
                    populate_vm(full_node_info, config)
                elif node['machine']['type'] == "Physical":
                    populate_device(full_node_info, config)

def populate_vm(nodeinfo, config):
    netbox_token        = get_parameter("netbox_token", secrets_filepath)
    vm_id               = get_netbox_vm_id(nodeinfo['hostname'], config)
    vm_netbox_interface_num = get_netbox_vm_interface_num(nodeinfo['hostname'], config)
    vm_rudder_interface_num = len(nodeinfo["networkInterfaces"])
    if vm_netbox_interface_num != vm_rudder_interface_num:
        delete_all_netbox_vm_interface(nodeinfo['hostname'], config)
        for network in nodeinfo["networkInterfaces"]:
            add_netbox_vm_interface(network, vm_id, config)
    elif vm_netbox_interface_num == vm_rudder_interface_num:
        for network in nodeinfo["networkInterfaces"]:
            patch_netbox_vm_interface(network, vm_id, config)

    for interface in nodeinfo["networkInterfaces"]:
        interface_id = get_netbox_vm_interface_id(vm_id, interface['name'], config)
        for ip in interface['ipAddresses']:
            add_netbox_ip(ip, interface_id, config)
    
    
def populate_device(nodeinfo, config):
    netbox_token        = get_parameter("netbox_token", secrets_filepath)
    device_id           = get_netbox_device_id(nodeinfo['hostname'], config)
    device_netbox_interface_num = get_netbox_device_interface_num(nodeinfo['hostname'], config)
    device_rudder_interface_num = len(nodeinfo["networkInterfaces"])
    if device_netbox_interface_num != device_rudder_interface_num:
        delete_all_netbox_device_interface(nodeinfo['hostname'], config)
        for network in nodeinfo["networkInterfaces"]:
            add_netbox_device_interface(network, device_id, config)
    elif device_netbox_interface_num == device_rudder_interface_num:
        for network in nodeinfo["networkInterfaces"]:
            patch_netbox_device_interface(network, device_id, config)
    
    for interface in nodeinfo["networkInterfaces"]:
        interface_id = get_netbox_device_interface_id(device_id, interface['name'], config)
        for ip in interface['ipAddresses']:
            add_netbox_ip(ip, interface_id, config)

"""
main function of program
"""
def main():
    get_nodes(config_file)

"""
Start
"""
main()
